<?php

namespace App\Domain\Request\Entity;

use App\Domain\Request\DTO\Request\Query\NewRequestDTO;
use App\Domain\Request\Status\Status;
use DateTimeImmutable;
use InvalidArgumentException;
use Ramsey\Uuid\Uuid;

class Request
{
    private readonly string $id;
    private readonly DateTimeImmutable $createdAt;
    private string $comment = '';
    private Status $status;

    private function __construct(
        private readonly string $message,
        private readonly string $topic,
        private readonly string $userId,
    ) {
        $this->id = Uuid::uuid4()->toString();
        $this->createdAt = new DateTimeImmutable('now');
        $this->status = Status::NEW;
    }

    public static function create(NewRequestDTO $requestDTO, string $userId): Request
    {
        self::validateUserData($requestDTO->topic, $userId, $requestDTO->message);

        return new Request($requestDTO->message, $requestDTO->topic, $userId);
    }

    /**
     * Идентификатор заявки
     *
     * @return string
     */
    public function getId(): string
    {
        return $this->id;
    }

    /**
     * Тема заявки
     *
     * @return string
     */
    public function getTopic(): string
    {
        return $this->topic;
    }

    public function getUserId(): string
    {
        return $this->userId;
    }

    /**
     * Сообщение пользователя в заявке
     *
     * @return string
     */
    public function getMessage(): string
    {
        return $this->message;
    }

    /**
     * Время создания
     *
     * @return DateTimeImmutable
     */
    public function getCreatedAt(): DateTimeImmutable
    {
        return $this->createdAt;
    }

    /**
     * Статус заявки
     *
     * @return Status
     */
    public function getStatus(): Status
    {
        return $this->status;
    }

    /**
     * Комментарий модератора
     *
     * @return string
     */
    public function getComment(): string
    {
        return $this->comment;
    }

    /**
     * Поставить новый статус
     *
     * @param Status $status
     * @param string $comment
     * @return void
     */
    public function setNewStatus(Status $status, string $comment): void
    {
        if (empty($comment)) {
            throw new InvalidArgumentException('comment should be not empty');
        }

        $this->status = $status;
        $this->comment = $comment;
    }

    /**
     * Валидация входных данных
     *
     * @param string $topic
     * @param string $userId
     * @param string $message
     * @return void
     */
    private static function validateUserData(string $topic, string $userId, string $message): void
    {
        if (empty($topic)) {
            throw new InvalidArgumentException('Topic is required field');
        }

        if (empty($userId)) {
            throw new InvalidArgumentException('UserId is required field');
        }

        if (empty($message)) {
            throw new InvalidArgumentException('Message is required field');
        }
    }
}
