<?php

namespace App\Domain\User\Entity;

use App\Domain\User\Enums\RoleCodes;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;
use InvalidArgumentException;
use Ramsey\Uuid\Uuid;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\UserInterface;

#[Entity]
#[Table(name: 'users')]
class User implements UserInterface, PasswordAuthenticatedUserInterface
{
    #[Id]
    #[Column(type: 'string')]
    public string $id;

    /**
     * Класс, описывающий пользователя
     *
     * @param string $email
     * @param string $password
     *
     * @param Role $role
     */
    public function __construct(
        #[Column(name: 'email', type: 'string', length: 255, unique: true)]
        private string $email,
        #[Column(name: 'password', type: 'string')]
        private string $password,
        #[ManyToOne(targetEntity: Role::class)]
        #[JoinColumn(name: 'role_id', referencedColumnName: 'id')]
        private readonly Role $role,
    ) {
        $this->id = Uuid::uuid4()->toString();

        $this->validateFields();
    }

    /**
     * Идентификатор пользователя
     *
     * @return string
     */
    public function getId(): string
    {
        return $this->id;
    }

    /**
     * Почта пользователя
     *
     * @return string
     */
    public function getEmail(): string
    {
        return $this->email;
    }

    /**
     * Роль пользователя
     *
     * @return RoleCodes
     */
    public function getRole(): RoleCodes
    {
        return $this->role->getRoleCode();
    }

    /**
     * Изменить пароль пользователя
     *
     * @param string $password
     * @return void
     */
    public function setPassword(string $password): void
    {
        $this->password = $password;
    }

    /**
     * Список ролей пользователя
     *
     * @return array|string[]
     */
    public function getRoles(): array
    {
        return [
            $this->getRole()->value,
        ];
    }

    /**
     * Удалить все чувствительные данные
     *
     * @return void
     */
    public function eraseCredentials(): void
    {
        // Не реализовано, так как возможен кейс, что пользователь класса запишет
        // объект без данных в бд
    }

    /**
     * Получить идентификатор пользователя
     *
     * @return string
     */
    public function getUserIdentifier(): string
    {
        return $this->id;
    }

    /**
     * Валидация введенных пользователем данных
     *
     * @return void
     */
    private function validateFields(): void
    {
        if (empty($this->email)) {
            throw new InvalidArgumentException("Email is required field");
        }

        if (empty($this->password)) {
            throw new InvalidArgumentException("Password is required field");
        }

        if (!filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
            throw new InvalidArgumentException("Field must be valid email");
        }
    }

    public function getPassword(): ?string
    {
        return $this->password;
    }
}
