<?php

namespace App\Infrastructure\Request\Reader;

use App\Domain\Request\DTO\PaginateQuery;
use App\Domain\Request\DTO\PaginationDataDTO;
use App\Domain\Request\DTO\Request\Query\GetRequestsDTO;
use App\Domain\Request\DTO\Request\Response\RequestDTO;
use App\Domain\Request\Interfaces\RequestReaderInterface;
use App\Domain\Request\Status\Status;
use DateTime;
use Doctrine\DBAL\Exception;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManagerInterface;
use InvalidArgumentException;

class RequestReader implements RequestReaderInterface
{
    private const REQUESTS_TABLE_NAME = 'requests';

    public function __construct(private readonly EntityManagerInterface $entityManager)
    {
    }

    /**
     * Все заявки пользователя
     *
     * @param GetRequestsDTO $request
     * @return PaginationDataDTO
     * @throws Exception
     */
    public function getRequests(GetRequestsDTO $request): PaginationDataDTO
    {
        $queryBuilder = $this->getQueryToRequests('r')
            ->leftJoin('r', 'users', 'u', 'u.id = r.user_id');

        $this->parseRequestToQueryCondition($queryBuilder, $request);
        return $this->getPaginateDTO($queryBuilder, $request->pagination);
    }

    /**
     * Получить заявку по идентификатору
     *
     * @param string $id
     * @param string|null $userId
     * @return RequestDTO|null
     * @throws Exception
     * @throws \Exception
     */
    public function getRequestById(string $id, ?string $userId = null): ?RequestDTO
    {
        if (empty($id)) {
            throw new InvalidArgumentException('Id cannot be empty');
        }

        $queryBuilder = $this->getQueryToRequests('r')
            ->leftJoin('r', 'users', 'u', 'u.id = r.user_id')
            ->setMaxResults(1);

        if ($userId !== null) {
            $queryBuilder->where($queryBuilder->expr()->eq('r.user_id', ":userId"))
                ->setParameter('userId', $userId);
        }

        $queryBuilder->where($queryBuilder->expr()->eq('r.id', ':id'))
            ->setParameter('id', $id);


        $response = $queryBuilder->fetchAssociative();

        if (!$response) {
            return null;
        }

        return $this->getRequestDTOByArray($response);
    }

    /**
     * Получить запрос к таблице запросов
     *
     * @param array $select
     * @param string|null $alias
     * @return QueryBuilder
     */
    private function getQueryToRequests(string $alias = null): QueryBuilder
    {
        $select = [
            'r.id',
            'r.message',
            'r.topic',
            'r.user_id',
            'r.created_at',
            'r.status',
            'r.comment',
            'u.email'
        ];
        $queryBuilder = $this->entityManager->getConnection()->createQueryBuilder();

        $queryBuilder
            ->select($select)
            ->from(self::REQUESTS_TABLE_NAME, $alias);

        return $queryBuilder;
    }

    /**
     * Загрузить условия в объект запроса
     *
     * @param QueryBuilder $queryBuilder
     * @param GetRequestsDTO $request
     * @return void
     */
    private function parseRequestToQueryCondition(QueryBuilder $queryBuilder, GetRequestsDTO $request): void
    {
        if (!empty($request->topic)) {
            $queryBuilder->where($queryBuilder->expr()->eq('r.topic', ':topic'));
            $queryBuilder->setParameter('topic', $request->topic);
        }

        if (isset($request->status) || $request->status !== null) {
            $queryBuilder->where($queryBuilder->expr()->eq('r.status', ":status"));
            $queryBuilder->setParameter('status', $request->status);
        }

        if (!empty($request->userId)) {
            $queryBuilder->where($queryBuilder->expr()->eq('r.user_id', ":user_id"));
            $queryBuilder->setParameter('user_id', $request->userId);
        }
    }

    /**
     * Получить DTO из запроса на чтение
     *
     * @param QueryBuilder $queryBuilder
     * @param PaginateQuery $paginateQuery
     * @return PaginationDataDTO
     * @throws Exception
     * @throws \Exception
     */
    private function getPaginateDTO(QueryBuilder $queryBuilder, PaginateQuery $paginateQuery): PaginationDataDTO
    {
        $items = [];

        $cloneQuery = clone $queryBuilder;

        $queryBuilder->setMaxResults($paginateQuery->perPage);
        $queryBuilder->setFirstResult($paginateQuery->getOffset());

        $cloneQuery->select('COUNT(r.id)');
        $count = $cloneQuery->executeQuery()->fetchOne();
        $result = $queryBuilder->executeQuery()->fetchAllAssociative();

        foreach ($result as $item) {
            $items[] = $this->getRequestDTOByArray($item);
        }
        $lastPage = ceil(($count / $paginateQuery->perPage));

        return new PaginationDataDTO($count, $lastPage, $paginateQuery->page, $items);
    }

    /**
     * Создать dto из массива
     *
     * @param array $item
     * @return RequestDTO
     * @throws \Exception
     */
    private function getRequestDTOByArray(array $item): RequestDTO
    {
        $status = Status::tryFrom($item['status']);
        $dateTime = new DateTime($item['created_at']);

        return new RequestDTO(
            $item['id'],
            $item['topic'],
            $item['message'],
            $item['user_id'],
            $item['email'],
            $dateTime,
            $status,
            $item['comment']
        );
    }
}