<?php

namespace User\Entity;

use App\Domain\User\Entity\Role;
use App\Domain\User\Entity\User;
use App\Domain\User\Enums\RoleCodes;
use PHPUnit\Framework\TestCase;

final class UserTest extends TestCase
{
    public function testCannotBeCreatedWithInvalidEmail(): void
    {
        $this->expectException(\InvalidArgumentException::class);

        /**
         * @var Role $mockRole
         */
        $mockRole = $this->getMockBuilder(Role::class)
            ->disableOriginalConstructor()
            ->getMock();

        new User('invalidEmail', 'password', $mockRole);
    }

    public function testCannotBeCreatedWithEmptyEmail(): void
    {
        $this->expectException(\InvalidArgumentException::class);

        /**
         * @var Role $mockRole
         */
        $mockRole = $this->getMockBuilder(Role::class)
            ->disableOriginalConstructor()
            ->getMock();

        new User('', 'password', $mockRole);
    }

    public function testCannotBeCreatedWithEmptyPassword(): void
    {
        $this->expectException(\InvalidArgumentException::class);

        /**
         * @var Role $mockRole
         */
        $mockRole = $this->getMockBuilder(Role::class)
            ->disableOriginalConstructor()
            ->getMock();

        new User('invalidEmail', '', $mockRole);
    }

    public function testCanBeCreated(): void
    {
        /**
         * @var Role $mockRole
         */
        $mockRole = $this->getMockBuilder(Role::class)
            ->setConstructorArgs([RoleCodes::BASIC_USER])
            ->getMock();

        $user = new User('validEmail@gmail.com', 'testpassword', $mockRole);

        $this->assertNotEmpty($user);
    }

    public function testReturnValidEmail(): void
    {
        /**
         * @var Role $mockRole
         */
        $mockRole = $this->getMockBuilder(Role::class)
            ->setConstructorArgs([RoleCodes::BASIC_USER])
            ->getMock();

        $user = new User('validEmail@gmail.com', 'testpassword', $mockRole);

        $this->assertSame($user->getEmail(), 'validEmail@gmail.com');
    }

    public function testReturnValidRoleList(): void
    {
        /**
         * @var Role $mockRole
         */
        $mockRole = $this->getMockBuilder(Role::class)
            ->disableOriginalConstructor()
            ->getMock();

        $mockRole->expects($this->once())
            ->method('getRoleCode')
            ->willReturn(RoleCodes::BASIC_USER);

        $user = new User('validEmail@gmail.com', 'testpassword', $mockRole);

        $this->assertSame([RoleCodes::BASIC_USER->value], $user->getRoles());
    }
}